// Inspired by https://fireship.io/lessons/firestore-advanced-usage-angularfire/
import { get } from 'svelte/store';
import { user } from './user';
import { addDoc, collection, deleteDoc, doc, getDoc, serverTimestamp, setDoc, updateDoc, } from 'firebase/firestore';
import { db } from '.';
export const getUid = () => {
    const u = get(user);
    return (u && u.uid) || 'anonymous'; // useful if allowing support messages to be saved by non-logged-in users
};
export function colRef(ref) {
    return typeof ref === 'string' ? collection(db, ref) : ref;
}
export function docRef(ref) {
    if (typeof ref === 'string') {
        const pathParts = ref.split('/');
        const documentId = pathParts.pop();
        const collectionString = pathParts.join('/');
        return doc(colRef(collectionString), documentId);
    }
    else {
        return ref;
    }
}
export async function getDocument(ref) {
    const docSnap = await getDoc(docRef(ref));
    return docSnap.exists() ? { ...docSnap.data(), id: docSnap.id } : null;
}
export function add(ref, data, opts = {}) {
    return addDoc(colRef(ref), {
        ...data,
        [opts.abbreviateMetadata ? 'ua' : 'updatedAt']: serverTimestamp(),
        [opts.abbreviateMetadata ? 'ca' : 'createdAt']: serverTimestamp(),
        [opts.abbreviateMetadata ? 'ub' : 'updatedBy']: getUid(),
        [opts.abbreviateMetadata ? 'cb' : 'createdBy']: getUid(),
    });
}
export async function set(ref, data, opts = {}) {
    const snap = await getDocument(ref);
    return await (snap
        ? update(ref, data)
        : setDoc(docRef(ref), {
            ...data,
            [opts.abbreviateMetadata ? 'ua' : 'updatedAt']: serverTimestamp(),
            [opts.abbreviateMetadata ? 'ca' : 'createdAt']: serverTimestamp(),
            [opts.abbreviateMetadata ? 'ub' : 'updatedBy']: getUid(),
            [opts.abbreviateMetadata ? 'cb' : 'createdBy']: getUid(),
        }, { merge: opts.merge }));
} // could split apart into set and upsert if desired, https://stackoverflow.com/questions/46597327/difference-between-set-with-merge-true-and-update
export function update(ref, data, opts = {}) {
    return updateDoc(docRef(ref), {
        ...data,
        [opts.abbreviateMetadata ? 'ua' : 'updatedAt']: serverTimestamp(),
        [opts.abbreviateMetadata ? 'ub' : 'updatedBy']: getUid(),
    }).catch((err) => {
        alert(err);
    });
}
export function deleteDocument(ref) {
    return deleteDoc(docRef(ref));
}
export async function docExists(ref) {
    return (await getDoc(docRef(ref))).exists();
}
