import { writable } from 'svelte/store';
import { getAuth, onAuthStateChanged, signOut } from 'firebase/auth';
import { doc, serverTimestamp, updateDoc } from 'firebase/firestore';
import { db, firebaseApp } from '.';
import { setCookie } from './helpers/cookies';
import { docStore } from './stores';
const userKey = 'firebase_user';
function createUserStore() {
    const { subscribe, set } = writable(null);
    let unsub;
    if (typeof window !== 'undefined') {
        const auth = getAuth(firebaseApp);
        let cached = null;
        cached = JSON.parse(localStorage.getItem(userKey));
        set(cached);
        onAuthStateChanged(auth, (u) => {
            if (u) {
                unsub && unsub();
                const userStore = docStore(`users/${u.uid}`, { log: true });
                unsub = userStore.subscribe((user) => {
                    if (user) {
                        set(user);
                        cacheUser(user);
                        denoteVisitOnce(user.uid);
                    }
                });
            }
            else {
                set(null);
                removeCachedUser();
            }
        }, (err) => console.error(err.message));
    }
    const signOutFn = async () => {
        const auth = getAuth(firebaseApp);
        unsub && unsub();
        await signOut(auth);
    };
    return {
        subscribe,
        signOut: signOutFn,
    };
}
export const user = createUserStore();
function cacheUser(user) {
    localStorage.setItem(userKey, JSON.stringify(user));
    const minimalUser = {
        displayName: user.displayName,
        email: user.email,
        photoURL: user.photoURL || null,
    }; // Cookies are limited to 4kb, about 1,000-4000 characters
    setCookie('user', JSON.stringify(minimalUser), { 'max-age': 31536000 });
}
function removeCachedUser() {
    localStorage.removeItem(userKey);
    const yesterday = new Date(new Date());
    yesterday.setDate(yesterday.getDate() - 1);
    setCookie('user', null, { expires: yesterday });
}
const denoteVisitOnce = (() => {
    let denoted = false;
    return async function (uid) {
        if (!denoted) {
            denoted = true;
            try {
                await updateDoc(doc(db, 'users', uid), { lastVisit: serverTimestamp() });
            }
            catch (err) {
                console.error(err);
            }
            return true;
        }
        else {
            return true;
        }
    };
})();
// OLD
// unsub = onSnapshot(doc(db, 'users', u.uid), (snapshot) => {
//   const user = snapshot.data() as IUser;
//   if (user) {
//     console.log('retrieved: ', user);
//   }
// });
